@@ -1,9 +1,13 @@
 import logging
 import os
 import time
+from mock import patch
+import pytest
+import kafka.codec
 
-from six.moves import xrange
-import six
+import pytest
+from kafka.vendor.six.moves import range
+from kafka.vendor import six
 
 from . import unittest
 from kafka import (
@@ -13,25 +17,29 @@ from kafka import (
 from kafka.consumer.base import MAX_FETCH_BUFFER_SIZE_BYTES
 from kafka.errors import (
     ConsumerFetchSizeTooSmall, OffsetOutOfRangeError, UnsupportedVersionError,
-    KafkaTimeoutError
+    KafkaTimeoutError, UnsupportedCodecError
 )
 from kafka.structs import (
     ProduceRequestPayload, TopicPartition, OffsetAndTimestamp
 )
 
-from test.fixtures import ZookeeperFixture, KafkaFixture
-from test.testutil import (
-    KafkaIntegrationTestCase, kafka_versions, random_string, Timer,
-    send_messages
-)
+from test.conftest import version
+from test.fixtures import ZookeeperFixture, KafkaFixture, random_string
+from test.testutil import KafkaIntegrationTestCase, kafka_versions, Timer
 
-def test_kafka_consumer(simple_client, topic, kafka_consumer_factory):
-    """Test KafkaConsumer
-    """
+
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
+def test_kafka_consumer(kafka_producer, topic, kafka_consumer_factory):
+    """Test KafkaConsumer"""
     kafka_consumer = kafka_consumer_factory(auto_offset_reset='earliest')
 
-    send_messages(simple_client, topic, 0, range(0, 100))
-    send_messages(simple_client, topic, 1, range(100, 200))
+    # TODO replace this with a `send_messages()` pytest fixture
+    # as we will likely need this elsewhere
+    for i in range(0, 100):
+        kafka_producer.send(topic, partition=0, value=str(i).encode())
+    for i in range(100, 200):
+        kafka_producer.send(topic, partition=1, value=str(i).encode())
+    kafka_producer.flush()
 
     cnt = 0
     messages = {0: set(), 1: set()}
@@ -47,6 +55,24 @@ def test_kafka_consumer(simple_client, topic, kafka_consumer_factory):
     kafka_consumer.close()
 
 
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
+def test_kafka_consumer_unsupported_encoding(
+        topic, kafka_producer_factory, kafka_consumer_factory):
+    # Send a compressed message
+    producer = kafka_producer_factory(compression_type="gzip")
+    fut = producer.send(topic, b"simple message" * 200)
+    fut.get(timeout=5)
+    producer.close()
+
+    # Consume, but with the related compression codec not available
+    with patch.object(kafka.codec, "has_gzip") as mocked:
+        mocked.return_value = False
+        consumer = kafka_consumer_factory(auto_offset_reset='earliest')
+        error_msg = "Libraries for gzip compression codec not found"
+        with pytest.raises(UnsupportedCodecError, match=error_msg):
+            consumer.poll(timeout_ms=2000)
+
+
 class TestConsumerIntegration(KafkaIntegrationTestCase):
     maxDiff = None
 
@@ -448,7 +474,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         )
 
         # Grab the first 195 messages
-        output_msgs1 = [ consumer1.get_message().message.value for _ in xrange(195) ]
+        output_msgs1 = [ consumer1.get_message().message.value for _ in range(195) ]
         self.assert_message_count(output_msgs1, 195)
 
         # The total offset across both partitions should be at 180
@@ -578,7 +604,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         # Grab the first 180 messages
         output_msgs1 = []
-        for _ in xrange(180):
+        for _ in range(180):
             m = next(consumer1)
             output_msgs1.append(m)
         self.assert_message_count(output_msgs1, 180)
@@ -594,7 +620,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         # 181-200
         output_msgs2 = []
-        for _ in xrange(20):
+        for _ in range(20):
             m = next(consumer2)
             output_msgs2.append(m)
         self.assert_message_count(output_msgs2, 20)
