@@ -2,7 +2,8 @@ import os
 import time
 import uuid
 
-from six.moves import range
+import pytest
+from kafka.vendor.six.moves import range
 
 from kafka import (
     SimpleProducer, KeyedProducer,
@@ -14,9 +15,11 @@ from kafka.errors import UnknownTopicOrPartitionError, LeaderNotAvailableError
 from kafka.producer.base import Producer
 from kafka.structs import FetchRequestPayload, ProduceRequestPayload
 
+from test.conftest import version
 from test.fixtures import ZookeeperFixture, KafkaFixture
 from test.testutil import KafkaIntegrationTestCase, kafka_versions, current_offset
 
+
 # TODO: This duplicates a TestKafkaProducerIntegration method temporarily
 # while the migration to pytest is in progress
 def assert_produce_request(client, topic, messages, initial_offset, message_ct,
@@ -32,6 +35,7 @@ def assert_produce_request(client, topic, messages, initial_offset, message_ct,
 
     assert current_offset(client, topic, partition) == initial_offset + message_ct
 
+
 def assert_produce_response(resp, initial_offset):
     """Verify that a produce response is well-formed
     """
@@ -39,6 +43,8 @@ def assert_produce_response(resp, initial_offset):
     assert resp[0].error == 0
     assert resp[0].offset == initial_offset
 
+
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
 def test_produce_many_simple(simple_client, topic):
     """Test multiple produces using the SimpleClient
     """
@@ -210,7 +216,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
 
-        producer = SimpleProducer(self.client, async=True, random_start=False)
+        producer = SimpleProducer(self.client, async_send=True, random_start=False)
         resp = producer.send_messages(self.topic, self.msg("one"))
         self.assertEqual(len(resp), 0)
 
@@ -229,7 +235,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         batch_interval = 5
         producer = SimpleProducer(
             self.client,
-            async=True,
+            async_send=True,
             batch_send_every_n=batch_messages,
             batch_send_every_t=batch_interval,
             random_start=False)
@@ -288,13 +294,14 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         producer.stop()
 
     def test_batched_simple_producer__triggers_by_time(self):
+        self.skipTest("Flakey test -- should be refactored or removed")
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
 
         batch_interval = 5
         producer = SimpleProducer(
             self.client,
-            async=True,
+            async_send=True,
             batch_send_every_n=100,
             batch_send_every_t=batch_interval,
             random_start=False)
@@ -426,7 +433,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer = KeyedProducer(self.client,
                                  partitioner=RoundRobinPartitioner,
-                                 async=True,
+                                 async_send=True,
                                  batch_send_every_t=1)
 
         resp = producer.send_messages(self.topic, self.key("key1"), self.msg("one"))
